'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

var _animationUtils = require('../utils/animation-utils');

var _theme = require('../theme');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } // Copyright (c) 2016 Uber Technologies, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

function _c(className) {
  var prefix = 'rv-treemap';
  if (!className) {
    return prefix;
  }
  return prefix + '__' + className;
}

var DEFAULT_SCALES = {
  color: {
    range: _theme.CONTINUOUS_COLOR_RANGE
  },
  opacity: {
    range: _theme.OPACITY_RANGE
  }
};

var FixedTreemapVis = function (_React$Component) {
  _inherits(FixedTreemapVis, _React$Component);

  _createClass(FixedTreemapVis, null, [{
    key: 'displayName',
    get: function get() {
      return 'FixedTreemapVis';
    }
  }, {
    key: 'propTypes',
    get: function get() {
      return {
        width: _react2.default.PropTypes.number.isRequired,
        height: _react2.default.PropTypes.number.isRequired,
        data: _react2.default.PropTypes.object.isRequired,
        mode: _react2.default.PropTypes.string.isRequired,
        padding: _react2.default.PropTypes.number.isRequired,
        animation: _animationUtils.AnimationPropType
      };
    }
  }, {
    key: 'defaultProps',
    get: function get() {
      return {
        mode: 'squarify',
        padding: 0,
        data: {
          children: []
        }
      };
    }
  }]);

  function FixedTreemapVis(props) {
    _classCallCheck(this, FixedTreemapVis);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(FixedTreemapVis).call(this, props));

    _this._renderLeaf = _this._renderLeaf.bind(_this);
    return _this;
  }

  /**
   * Walk through the tree and collect all values from the tree.
   * @param {Object} data Tree object.
   * @param {string} propName Property name.
   * @returns {Array} Flat array of values.
   * @private
   */


  _createClass(FixedTreemapVis, [{
    key: '_collectValuesFromTree',
    value: function _collectValuesFromTree(data, propName) {
      var result = [];
      if (data[propName]) {
        result.push(data[propName]);
      }
      if (Array.isArray(data.children)) {
        data.children.forEach(function getValuesFromChildren(child) {
          result = result.concat(this._collectValuesFromTree(child, propName));
        }, this);
      }
      return result;
    }

    /**
     * Get the scale function from the options and from the array.
     * @param {Object} scaleOptions Scale options that are passed in props.
     * @param {Array} defaultDomain Default domain.
     * @param {Array} defaultRange Default range.
     * @returns {*} Scale function or undefined if scaleOptions are not defined.
     * @private
     */

  }, {
    key: '_getScaleFunction',
    value: function _getScaleFunction(scaleOptions, defaultDomain, defaultRange) {
      var domain = undefined;
      var range = undefined;
      var scaleFn = undefined;
      if (scaleOptions) {
        domain = scaleOptions.domain;
        range = scaleOptions.range;
        scaleFn = scaleOptions.scaleFn;
      }
      if (!scaleFn) {
        if (!domain) {
          domain = defaultDomain;
        }
        if (!range) {
          range = defaultRange;
        }
        scaleFn = _d2.default.scale.linear().range(range).domain(domain);
      }
      return scaleFn;
    }
  }, {
    key: '_getColorScaleFunction',
    value: function _getColorScaleFunction(data) {
      var values = this._collectValuesFromTree(data, 'color');
      if (values.length) {
        var range = DEFAULT_SCALES.color.range;
        var domain = _d2.default.extent(values);
        var options = this.props.scales ? this.props.scales.color : null;
        return this._getScaleFunction(options, domain, range);
      }
      return function () {
        return _theme.DEFAULT_COLOR;
      };
    }
  }, {
    key: '_getOpacityScaleFunction',
    value: function _getOpacityScaleFunction(data) {
      var values = this._collectValuesFromTree(data, 'opacity');
      if (values.length) {
        var range = DEFAULT_SCALES.opacity.range;
        var domain = _d2.default.extent(values);
        var options = this.props.scales ? this.props.scales.opacity : null;
        return this._getScaleFunction(options, domain, range);
      }
    }

    /**
     * Create the list of nodes to render.
     * @returns {Array} Array of nodes.
     * @private
     */

  }, {
    key: '_getNodesToRender',
    value: function _getNodesToRender() {
      var data = this.props.data;
      var height = this.props.height;
      var width = this.props.width;
      var mode = this.props.mode;
      var padding = this.props.padding;
      var nodes = [];
      var layout = undefined;
      if (data) {
        layout = _d2.default.layout.treemap().padding(padding).mode(mode).sort(function (a, b) {
          return a.size - b.size;
        });
        nodes = layout.size([width, height]).value(function (d) {
          return d.size;
        }).sticky(true).nodes(data);
      }

      return nodes;
    }
  }, {
    key: '_renderLeaf',
    value: function _renderLeaf(node, i) {
      if (!i) {
        return null;
      }
      var data = this.props.data;

      var colorScale = this._getColorScaleFunction(data);
      var opacityScale = this._getOpacityScaleFunction(data);
      // Don't draw the background for the first item: it's a container
      // for the entire treemap.
      var background = colorScale ? colorScale(node.color) : null;
      var opacity = opacityScale ? opacityScale(node.opacity) : null;
      var color = null;
      if (background) {
        color = _d2.default.hsl(background).l > 0.57 ? '#222' : '#fff';
      }

      var width = Math.max(0, node.dx - 1);
      var height = Math.max(0, node.dy - 1);
      var style = (0, _animationUtils.getCSSAnimation)(this.props, {
        top: node.y + 'px',
        left: node.x + 'px',
        width: width + 'px',
        height: height + 'px',
        background: background,
        opacity: opacity,
        color: color
      });
      return _react2.default.createElement(
        'div',
        {
          key: i,
          className: _c('leaf'),
          style: style },
        _react2.default.createElement(
          'div',
          { className: 'rv-treemap__leaf__content' },
          node.title
        )
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props;
      var width = _props.width;
      var height = _props.height;

      var nodes = this._getNodesToRender();
      return _react2.default.createElement(
        'div',
        {
          className: _c(),
          style: {
            width: width + 'px',
            height: height + 'px'
          } },
        nodes.map(this._renderLeaf)
      );
    }
  }]);

  return FixedTreemapVis;
}(_react2.default.Component);

exports.default = FixedTreemapVis;